#include "precompile.h"

#include <google/protobuf/message.h>
#include <a8/mixedsession.h>
#include <a8/tcplistener.h>

#include "framework/cpp/netmsghandler.h"

#include "app.h"
#include "IMListener.h"
#include "jsondatamgr.h"
#include "ss_proto.pb.h"
#include "handlermgr.h"

class IMServerSession: public a8::MixedSession
{
public:

    virtual void DecodeUserPacket(char* buf, int& offset, unsigned int buflen) override
    {
        //packagelen + msgid + magiccode + msgbody
        //2 + 2 + 4+ xx + \0 + xx
        #if 1
        is_activite = true;
        #endif
        bool warning = false;
        while (buflen - offset >= sizeof(f8::PackHead)) {
            f8::PackHead* p = (f8::PackHead*)&buf[offset];
            if (p->magic_code == f8::MAGIC_CODE) {
                if (buflen - offset < sizeof(f8::PackHead) + p->packlen) {
                    break;
                }
                App::Instance()->AddSocketMsg(SF_IMServer,
                                              socket_handle,
                                              saddr,
                                              p->msgid,
                                              p->seqid,
                                              &buf[offset + sizeof(f8::PackHead)],
                                              p->packlen);
                offset += sizeof(f8::PackHead) + p->packlen;
            } else {
                warning = true;
                offset++;
                continue;
            }
        }

        if (warning) {
            a8::UdpLog::Instance()->Warning("收到imserver非法数据包", {});
        }
    }

    virtual void OnRawHttpGet(const std::string& url, const std::string& querystr,
                              std::string& response) override
    {
        App::Instance()->AddIMMsg(IM_ExecGM,
                                  a8::XParams()
                                  .SetSender(socket_handle)
                                  .SetParam1(url)
                                  .SetParam2(querystr)
                                  .SetParam3(saddr));
    }

    virtual void OnDisConnect() override
    {
        App::Instance()->AddIMMsg(IM_IMServerSocketDisconnect,
                                  a8::XParams()
                                  .SetSender(socket_handle)
                                  .SetParam1(1));
    }

};

static void CreateIMServerSocket(a8::TcpSession **p)
{
    *p = new IMServerSession();
}

static void GSListeneron_error(a8::TcpListener*, int type, int errorid)
{
    a8::UdpLog::Instance()->Debug("IMListeneron_error %d %d", {type, errorid});
}

void IMListener::Init()
{
    tcp_listener_ = new a8::TcpListener();
    tcp_listener_->on_create_client_socket = CreateIMServerSocket;
    tcp_listener_->on_error = GSListeneron_error;

    tcp_listener_->bind_address = "0.0.0.0";
    tcp_listener_->bind_port = JsonDataMgr::Instance()->GetConf()->At("imserver_port")->AsXValue();
    tcp_listener_->Open();
}

void IMListener::UnInit()
{
    delete tcp_listener_;
    tcp_listener_ = nullptr;
}

void IMListener::ForwardTargetConnMsg(f8::MsgHdr& hdr)
{
    char* buff = (char*)malloc(sizeof(f8::PackHead) + hdr.buflen);
    f8::PackHead* head = (f8::PackHead*)buff;
    head->packlen = hdr.buflen;
    head->msgid = hdr.msgid;
    head->seqid = hdr.seqid;
    head->magic_code = f8::MAGIC_CODE;
    head->ext_len = hdr.buflen >> 16;
    if (hdr.buflen > 0) {
        memmove(buff + sizeof(f8::PackHead), hdr.buf, hdr.buflen);
    }

    tcp_listener_->SendClientMsg(hdr.socket_handle, buff, sizeof(f8::PackHead) + head->packlen);
    free(buff);
}

void IMListener::SendText(unsigned short sockhandle, const std::string& text)
{
    tcp_listener_->SendClientMsg(sockhandle, text.data(), text.size());
}

void IMListener::ForceCloseClient(unsigned short sockhandle)
{
    tcp_listener_->ForceCloseClient(sockhandle);
}

void IMListener::MarkClient(unsigned short sockhandle, bool is_active)
{
    tcp_listener_->MarkClient(sockhandle, is_active);
}

void IMListener::_SS_Ping(f8::MsgHdr& hdr, const ss::SS_Ping& msg)
{
    ss::SS_Pong pongmsg;
    SendMsg(hdr.socket_handle, pongmsg);
}
