#pragma once

#include <a8/timer_attacher.h>

#include "framework/cpp/protoutils.h"
#include "framework/cpp/netmsghandler.h"

#include "perfmonitor.h"

namespace a8
{
    class AsyncTcpClient;
}

struct timer_list;
class MSConn
{
 public:
    int instance_id = 0;
    std::string remote_ip;
    int remote_port = 0;
    a8::tick_t last_pong_tick = 0;

 public:
    void Init(int instance_id, const std::string& remote_ip, int remote_port);
    void UnInit();

    void Open();
    void Close();
    bool Connected();

    template <typename T>
    void SendMsg(T& msg)
    {
        static int msgid = f8::Net_GetMessageId(msg);
        f8::Net_SendMsg(tcp_client_, 0, msgid, msg);
        ++PerfMonitor::Instance()->ms_send_times;
        #ifdef DEBUG
        f8::DumpMsgToLog(msg, "<<<<<<<MSC ");
        #endif
    }
    void SendMsg(int msgid, ::google::protobuf::Message& msg)
    {
        f8::Net_SendMsg(tcp_client_, 0, msgid, msg);
        ++PerfMonitor::Instance()->ms_send_times;
        #ifdef DEBUG
        f8::DumpMsgToLog(msg, "<<<<<<<IMC ");
        #endif
    }

    void OnConnectSync();
    void OnDisconnectSync();

 private:
    void on_error(a8::AsyncTcpClient* sender, int errorId);
    void on_connect(a8::AsyncTcpClient* sender);
    void on_disconnect(a8::AsyncTcpClient* sender);
    void on_socketread(a8::AsyncTcpClient* sender, char* buf, unsigned int len);

    void CheckAlive();
    void SyncIMServerList();
    void ReportServerInfo();

 private:
    char *recv_buff_ = nullptr;
    unsigned int recv_bufflen_ = 0;
    a8::AsyncTcpClient* tcp_client_ = nullptr;

    a8::TimerAttacher timer_attacher_;
};
