#include "precompile.h"

#include <a8/timer.h>
#include <a8/ioloop.h>

#include "perfmonitor.h"
#include "app.h"
#include "playermgr.h"
#include "dbhelper.h"

#include "framework/cpp/dbpool.h"

static void SavePerfLog()
{
    a8::UdpLog::Instance()->Info
        ("max_run_delay_time:%d max_timer_idle:%d "
         "in_data_size:%d out_data_size:%d msgnode_size:%d read_count:%d max_login_time:%d "
         "login_ok_time:%d login_error_time:%d online_num:%d watch_num:%d cache_users_num:%d "
         "send_ws_count:%d db.total_query_num:%d db.exec_query_num:%d db.run_loop_num:%d "
         "ms_send_times:%d ms_recv_times:%d ",
         {
          PerfMonitor::Instance()->max_run_delay_time,
          PerfMonitor::Instance()->max_timer_idle,
          PerfMonitor::Instance()->in_data_size,
          PerfMonitor::Instance()->out_data_size,
          App::Instance()->msgnode_size_,
          PerfMonitor::Instance()->read_count,
          PerfMonitor::Instance()->max_login_time,
          PerfMonitor::Instance()->login_ok_times,
          PerfMonitor::Instance()->login_error_times,
          PlayerMgr::Instance()->OnlineNum(),
          PlayerMgr::Instance()->WatchPlayerNum(),
          DBHelper::Instance()->cache_users_hash.size(),
          PerfMonitor::Instance()->send_ws_count,
          (long long)f8::DBPool::Instance()->total_query_num,
          (long long)f8::DBPool::Instance()->exec_query_num,
          (long long)f8::DBPool::Instance()->run_loop_num,
          PerfMonitor::Instance()->ms_send_times,
          PerfMonitor::Instance()->ms_recv_times
         });
    a8::UdpLog::Instance()->Info
        ("run_times:%d timer_times:%d event_times:%d free_times:%d "
         "shutdown_times:%d connect_times:%d close_times:%d "
         "send_times:%d recv_times:%d error_times:%d immsg_times:%d",
         {
          (long long)a8::IoLoop::Instance()->run_times,
          (long long)a8::IoLoop::Instance()->timer_times,
          (long long)a8::IoLoop::Instance()->event_times,
          (long long)a8::IoLoop::Instance()->free_times,
          (long long)a8::IoLoop::Instance()->shutdown_times,
          (long long)a8::IoLoop::Instance()->connect_times,
          (long long)a8::IoLoop::Instance()->close_times,
          (long long)a8::IoLoop::Instance()->send_times,
          (long long)a8::IoLoop::Instance()->recv_times,
          (long long)a8::IoLoop::Instance()->error_times,
          (long long)a8::IoLoop::Instance()->immsg_times
         });
    PerfMonitor::Instance()->max_run_delay_time = 0;
    PerfMonitor::Instance()->max_timer_idle = 0;
    PerfMonitor::Instance()->max_login_time = 0;
}

void PerfMonitor::Init()
{
    {
        int perf_log_time = 1000 * 60 * 5;
        a8::Timer::Instance()->AddRepeatTimer(perf_log_time,
                                              a8::XParams(),
                                              [] (const a8::XParams& param)
                                              {
                                                  SavePerfLog();
                                              });
    }
}

void PerfMonitor::UnInit()
{

}
