#pragma once

#include "cs_proto.pb.h"
#include "ss_proto.pb.h"
#include "ss_msgid.pb.h"
#include "WSListener.h"
#include "dbengine.h"

struct Friend;
struct timer_list;
class Player
{
 public:
    enum { HID = HID_Player };

 public:
    int socket_handle = 0;
    a8::TimerAttacher timer_attacher;

    Friend myself;
    RoleData role_data;
    int gameid = 0;
    int channel = 0;
    long ip_saddr = 0;
    int account_registertime = 0;
    int red_point_flags_ = 0;

    int chat_channel = -1;
    std::string private_target;

    long long world_channel_last_id = 0;
    long long guild_channel_last_id = 0;

 public:
    void Init();
    void UnInit();

    void Deserialize(const ss::MFUserDB& friend_db);
    void Serialize(ss::MFUserDB& friend_db);

    template <typename T>
    void SendMsg(T& msg)
    {
        WSListener::Instance()->SendToClient(socket_handle, 0, msg);
    }

    template <typename T>
    void SendSSMsg(const Friend& friend_data, T& msg)
    {
        static int msgid = f8::Net_GetMessageId(msg);
        InternalSendSSMsg(friend_data, msgid, msg);
    }

    template <typename T>
    void SendSSMsg(const std::vector<Friend*> friend_list, T& msg)
    {
        static int msgid = f8::Net_GetMessageId(msg);
        for (auto& friend_data : friend_list) {
            InternalSendSSMsg(*friend_data, msgid, msg);
        }
    }

    template <typename T>
    void BoradcastSSMsg(const T& msg)
    {
        static int msgid = f8::Net_GetMessageId(msg);
        InternalSendSSMsg(myself, msgid, msg);
        BoradcastExcludeSSMsg(msg);
    }

    template <typename T>
    void BoradcastExcludeSSMsg(const T& msg)
    {
        static int msgid = f8::Net_GetMessageId(msg);
        for (auto& pair : friend_hash_) {
            InternalSendSSMsg(pair.second, msgid, msg);
        }
    }

   template <typename T>
    void SendGSMsg(T& msg)
    {
        static int msgid = f8::Net_GetMessageId(msg);
        SendGSMsg(msgid, msg);
    }

    void _CMPing(f8::MsgHdr& hdr, const cs::CMPing& msg);
    void _CMUpdateUserInfo(f8::MsgHdr& hdr, const cs::CMUpdateUserInfo& msg);
    void _CMUpdateTempCustomData(f8::MsgHdr& hdr, const cs::CMUpdateTempCustomData& msg);
    void _CMFriendList(f8::MsgHdr& hdr, const cs::CMFriendList& msg);
    void _CMFriendApply(f8::MsgHdr& hdr, const cs::CMFriendApply& msg);
    void _CMFriendApplyList(f8::MsgHdr& hdr, const cs::CMFriendApplyList& msg);
    void _CMFriendAgree(f8::MsgHdr& hdr, const cs::CMFriendAgree& msg);
    void _CMFriendRefuse(f8::MsgHdr& hdr, const cs::CMFriendRefuse& msg);
    void _CMFriendDelete(f8::MsgHdr& hdr, const cs::CMFriendDelete& msg);
    void _CMFriendBlackList(f8::MsgHdr& hdr, const cs::CMFriendBlackList& msg);
    void _CMFriendAddBlack(f8::MsgHdr& hdr, const cs::CMFriendAddBlack& msg);
    void _CMFriendDeleteBlack(f8::MsgHdr& hdr, const cs::CMFriendDeleteBlack& msg);
    void _CMQueryUserStatus(f8::MsgHdr& hdr, const cs::CMQueryUserStatus& msg);
    void _CMRecommandFriend(f8::MsgHdr& hdr, const cs::CMRecommandFriend& msg);
    void _CMFriendIdList(f8::MsgHdr& hdr, const cs::CMFriendIdList& msg);

    void _CMSendChatMsg(f8::MsgHdr& hdr, const cs::CMSendChatMsg& msg);
    void _CMReadMsgAndOpenChatNotify(f8::MsgHdr& hdr, const cs::CMReadMsgAndOpenChatNotify& msg);
    void _CMSetCurrPrivateChatTarget(f8::MsgHdr& hdr, const cs::CMSetCurrPrivateChatTarget& msg);
    void _CMCloseChatNotify(f8::MsgHdr& hdr, const cs::CMCloseChatNotify& msg);
    void _CMSendCustomMsg(f8::MsgHdr& hdr, const cs::CMSendCustomMsg& msg);
    void _CMDirtyWordCheck(f8::MsgHdr& hdr, const cs::CMDirtyWordCheck& msg);

    void _CMTeamCreate(f8::MsgHdr& hdr, const cs::CMTeamCreate& msg);
    void _CMTeamJoin(f8::MsgHdr& hdr, const cs::CMTeamJoin& msg);
    void _CMTeamAgree(f8::MsgHdr& hdr, const cs::CMTeamAgree& msg);
    void _CMTeamKick(f8::MsgHdr& hdr, const cs::CMTeamKick& msg);
    void _CMTeamQuit(f8::MsgHdr& hdr, const cs::CMTeamQuit& msg);
    void _CMTeamDismiss(f8::MsgHdr& hdr, const cs::CMTeamDismiss& msg);
    void _CMTeamRename(f8::MsgHdr& hdr, const cs::CMTeamRename& msg);

    void _CMGuildInfo(f8::MsgHdr& hdr, const cs::CMGuildInfo& msg);
    void _CMGuildCreate(f8::MsgHdr& hdr, const cs::CMGuildCreate& msg);
    void _CMGuildJoin(f8::MsgHdr& hdr, const cs::CMGuildJoin& msg);
    void _CMGuildAgree(f8::MsgHdr& hdr, const cs::CMGuildAgree& msg);
    void _CMGuildRefuse(f8::MsgHdr& hdr, const cs::CMGuildRefuse& msg);
    void _CMGuildKick(f8::MsgHdr& hdr, const cs::CMGuildKick& msg);
    void _CMGuildQuit(f8::MsgHdr& hdr, const cs::CMGuildQuit& msg);
    void _CMGuildDismiss(f8::MsgHdr& hdr, const cs::CMGuildDismiss& msg);
    void _CMGuildChange(f8::MsgHdr& hdr, const cs::CMGuildChange& msg);
    void _CMGuildSearch(f8::MsgHdr& hdr, const cs::CMGuildSearch& msg);
    void _CMGuildRank(f8::MsgHdr& hdr, const cs::CMGuildRank& msg);
    void _CMGuildMemberList(f8::MsgHdr& hdr, const cs::CMGuildMemberList& msg);
    void _CMGuildApplyList(f8::MsgHdr& hdr, const cs::CMGuildApplyList& msg);
    void _CMGuildLog(f8::MsgHdr& hdr, const cs::CMGuildLog& msg);
    void _CMGuildMemberSetJob(f8::MsgHdr& hdr, const cs::CMGuildMemberSetJob& msg);
    void _CMGuildAgreeInvite(f8::MsgHdr& hdr, const cs::CMGuildAgreeInvite& msg);
    void _CMGuildGainExp(f8::MsgHdr& hdr, const cs::CMGuildGainExp& msg);
    void _CMGuildSearchMember(f8::MsgHdr& hdr, const cs::CMGuildSearchMember& msg);

    void _SS_IM_FriendAgreeRequest(f8::MsgHdr& hdr, const ss::SS_IM_FriendAgreeRequest& msg);
    void _SS_IM_FriendDeleteRequest(f8::MsgHdr& hdr, const ss::SS_IM_FriendDeleteRequest& msg);
    void _SS_IM_FriendApply(f8::MsgHdr& hdr, const ss::SS_IM_FriendApply& msg);

    void ReLogin(f8::MsgHdr& hdr, const cs::CMLogin& msg);
    void NotifyOnline();
    void NotifyOffline();
    void NotifyUserInfoUpdate(Friend* friend_data);
    void FillIMMsgConext(ss::MFIMMsgConext* context);
    void Update(long long tick);
    void FillMFUserInfo(cs::MFUserInfo* user_info);
    std::set<std::string>& GetExcludeAccountIds();
    Friend* GetFriendById(const std::string& friend_id);
    Friend* GetBlackListById(const std::string& friend_id);
    void UpdateGuildData(long long guild_id, int guild_job);
    void FillSMLogin(cs::SMLogin& respmsg);
    void UpdateGuildRedPoint(long long guild_id, int has_apply);
    void AddGuildApplyed(long long guild_id);
    void RemoveGuildApplyed(long long guild_id);

    const std::string AccountId();
    const std::string SessionId();
    const std::string NickName();
    const std::string AvatarUrl();
    long long GuildId();
    int VipLv();
    int Head();
    int Sex();
    int GuildJob();
    int GetFriendNum();
    long long GetDBPrivateChatLastId();
    long long IncDBPrivateChatLastId();

    void SaveToDB(a8::XParams param, f8::AsyncDBOnOkFunc on_ok, f8::AsyncDBOnErrorFunc on_error);
    void MarkNewMsg();
    void SyncRedPoint();

private:
    void FillFriendList(::google::protobuf::RepeatedPtrField< ::cs::MFUserInfo >* friend_list);
    void FillBlackList(::google::protobuf::RepeatedPtrField< ::cs::MFUserInfo >* black_list);
    void ForwardGroupCMMsg(f8::MsgHdr& hdr, long long hash_code);
    void ForwardGuildCMMsg(f8::MsgHdr& hdr, long long guild_id);
    void SendGSMsg(int msgid, ::google::protobuf::Message& msg);
    void FillApplyList(const cs::MFPaging& paging, cs::SMFriendApplyList& respmsg);
    void MarkDirty();
    void OnDataVersion1Change();
    void OnTempCustomDataChange();
    void InternalSendSSMsg(const Friend& friend_data,
                           int msgid,
                           ::google::protobuf::Message& msg);
    void InternalUpdateUserInfo();
    int AddFriend(Friend* friendobj);
    void AddBlackList(Friend* friendobj);
    void RemoveFriend(const std::string& account_id, bool need_sync);
    void PushFriendList();
    void PushBlackList();
    void SyncOtherFriend();
    void ProcessEventTimerFunc();
    void OnFetchEvent(const f8::DataSet* data_set);
    void ProcessEvent();
    void OnFriendAgreeEvent(Event& event);
    void OnFriendDeleteEvent(Event& event);
    void OnGuildRefuseEvent(Event& event);
    bool CanAddFriend(const std::string& account_id);
    void OnDailyReset();
    void RecalcRedPoint();
    void ClearApplyByIdx(long long idx);
    void ClearApplyBySenderId(const std::string& sender_id);
    void RefreshFriendData();
    void RemoveHandledApply();
    void CombineRepeatApply();
    void QueryUserOnline(std::vector<std::string> account_ids);
    void QueryUserFromDB();
    void ShowErrorMsg(const std::string& msg);
    void InternalRemoveFriend(const std::string& account_id, bool need_sync, bool auto_delete);
    void SyncGuildMemberInfo();
    void SyncGuildRedPoint();
    void SyncGuildNewApply(long long guild_id);
    bool IsValidChatChannel(int chat_channel);
    void SyncPrivateChatRedPoint();

private:
    bool dirty_ = false;
    timer_list* dirty_timer_ = nullptr;
    timer_list* update_user_info_timer_ = nullptr;
    long long last_apply_idx_ = 0;
    long long last_event_idx_ = 0;
    long long curr_max_event_idx_ = 0;
    bool event_fetching_ = false;
    long long last_run_tick_ = 0;
    std::map<long long, Event> event_hash_;
    std::set<std::string> exclude_account_ids_;
    std::string user_sign_;
    int last_create_guild_time_ = 0;
    long long db_private_chat_last_id_ = 0;

    std::map<std::string, Friend*> friend_hash_;
    std::map<std::string, Friend*> black_hash_;
    std::map<long long, FriendApply*> apply_hash_;
    std::map<long long, long long> applyed_guild_hash_;
};
