#include "precompile.h"

#include <a8/mutable_xobject.h>

#include "handlermgr.h"

#include "app.h"
#include "IMListener.h"
#include "perfmonitor.h"
#include "guildmgr.h"

static void _GMOpsSelfChecking(f8::JsonHttpRequest* request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    request->resp_xobj->SetVal("healthy", 1);
    request->resp_xobj->SetVal("max_rundelay", PerfMonitor::Instance()->max_run_delay_time);
    request->resp_xobj->SetVal("max_timer_idle", PerfMonitor::Instance()->max_timer_idle);
}

static void _GMOpsReload(f8::JsonHttpRequest* request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    a8::UdpLog::Instance()->Warning("reload config files", {});
}

void HandlerMgr::Init()
{
    RegisterNetMsgHandlers();
    RegisterGMMsgHandler("Ops@selfChecking", _GMOpsSelfChecking);
    RegisterGMMsgHandler("Ops@reload", _GMOpsReload);
    RegisterGMMsgHandler("Guild@create",
                         [] (f8::JsonHttpRequest* request)
                         {
                             GuildMgr::Instance()->__GuildCreate(request);
                         });
    RegisterGMMsgHandler("Guild@search",
                         [] (f8::JsonHttpRequest* request)
                         {
                             GuildMgr::Instance()->__GuildSearch(request);
                         });
    RegisterGMMsgHandler("Guild@update",
                         [] (f8::JsonHttpRequest* request)
                         {
                             GuildMgr::Instance()->__GuildUpdate(request);
                         });
    RegisterGMMsgHandler("Guild@rename",
                         [] (f8::JsonHttpRequest* request)
                         {
                             GuildMgr::Instance()->__GuildRename(request);
                         });
    RegisterGMMsgHandler("Guild@rank",
                         [] (f8::JsonHttpRequest* request)
                         {
                             GuildMgr::Instance()->__GuildRank(request);
                         });
    RegisterGMMsgHandler("Guild@confirmed",
                         [] (f8::JsonHttpRequest* request)
                         {
                             GuildMgr::Instance()->__GuildConfirmed(request);
                         });
    RegisterGMMsgHandler("Guild@dirtyWordCheck",
                         [] (f8::JsonHttpRequest* request)
                         {
                             GuildMgr::Instance()->__GuildDirtyWordCheck(request);
                         });
    RegisterGMMsgHandler("Guild@genTestData",
                         [] (f8::JsonHttpRequest* request)
                         {
                             GuildMgr::Instance()->__GenTestData(request);
                         });
}

void HandlerMgr::UnInit()
{
}

void HandlerMgr::RegisterNetMsgHandlers()
{
}

void HandlerMgr::ProcGMMsg(unsigned long saddr, int sockhandle,
                           const std::string& url, const std::string& querystr)
{
    if (url != "/webapp/index.php") {
        IMListener::Instance()->SendText(sockhandle, a8::HttpResponse(404, ""));
        return;
    }

    a8::HTTPRequest request;
    a8::ParserUrlQueryString(querystr.c_str(), request);

    std::string msgname = a8::Get(request, "c").GetString() + "@" + a8::Get(request, "a").GetString();
    auto itr = gmhandlers_.find(msgname);
    if (itr != gmhandlers_.end()) {
        f8::JsonHttpRequest* request = new f8::JsonHttpRequest;
        request->saddr = saddr;
        request->socket_handle = sockhandle;
        request->query_str = querystr;
        request->request.ReadFromUrlQueryString(querystr);
        request->resp_xobj->SetVal("errcode", 0);
        request->resp_xobj->SetVal("errmsg", "");
        itr->second(request);

        if (!request->pending){
            std::string response;
            request->resp_xobj->ToJsonStr(response);
            IMListener::Instance()->SendText(sockhandle, a8::HttpResponse(response));

            delete request;
        }
    } else {
        IMListener::Instance()->SendText(sockhandle, a8::HttpResponse("{}"));
    }
}

void HandlerMgr::RegisterGMMsgHandler(const std::string& msgname,
                                      void (*handler)(f8::JsonHttpRequest*))
{
    gmhandlers_[msgname] = handler;
}
